# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
import typing_extensions
from ...core.serialization import FieldMetadata
from .sheet_access import SheetAccess
from ...property.types.property import Property
from ...commons.types.action import Action
from .sheet_constraint import SheetConstraint
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SheetConfig(UniversalBaseModel):
    """
    Describes shape of data as well as behavior

    Examples
    --------
    from flatfile.property import Property_String
    from flatfile.sheets import SheetConfig

    SheetConfig(
        name="Contacts",
        slug="contacts",
        fields=[
            Property_String(
                key="firstName",
                label="First Name",
            ),
            Property_String(
                key="lastName",
                label="Last Name",
            ),
            Property_String(
                key="email",
                label="Email",
            ),
        ],
        mapping_confidence_threshold=0.5,
    )
    """

    name: str = pydantic.Field()
    """
    The name of your Sheet as it will appear to your end users.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A sentence or two describing the purpose of your Sheet.
    """

    slug: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for your Sheet.
    """

    readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    A boolean specifying whether or not this sheet is read only. Read only sheets are not editable by end users.
    """

    allow_additional_fields: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="allowAdditionalFields")
    ] = pydantic.Field(default=None)
    """
    Allow end users to add fields during mapping.
    """

    mapping_confidence_threshold: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="mappingConfidenceThreshold")
    ] = pydantic.Field(default=None)
    """
    The minimum confidence required to automatically map a field
    """

    access: typing.Optional[typing.List[SheetAccess]] = pydantic.Field(default=None)
    """
    Control Sheet-level access for all users.
    """

    fields: typing.List[Property] = pydantic.Field()
    """
    Where you define your Sheet’s data schema.
    """

    actions: typing.Optional[typing.List[Action]] = pydantic.Field(default=None)
    """
    An array of actions that end users can perform on this Sheet.
    """

    metadata: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(default=None)
    """
    Useful for any contextual metadata regarding the schema. Store any valid json
    """

    constraints: typing.Optional[typing.List[SheetConstraint]] = pydantic.Field(default=None)
    """
    An array of constraints that end users can perform on this Sheet.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
