# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from ...commons.types.snapshot_id import SnapshotId
import pydantic
import typing_extensions
from ...commons.types.sheet_id import SheetId
from ...core.serialization import FieldMetadata
import typing
from .snapshot_summary import SnapshotSummary
import datetime as dt
from ...commons.types.user_id import UserId
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Snapshot(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from flatfile.property import Property_String
    from flatfile.sheets import SheetConfig
    from flatfile.snapshots import Snapshot, SnapshotSummary, SummarySection

    Snapshot(
        id="us_ss_YOUR_ID",
        sheet_id="us_sh_YOUR_ID",
        label="My snapshot",
        summary=SnapshotSummary(
            created_since=SummarySection(
                total=0,
            ),
            updated_since=SummarySection(
                total=5,
                by_field={"lastName": 5},
            ),
            deleted_since=SummarySection(
                total=5,
                by_field={"firstName": 1},
            ),
            schema_diff={
                "firstName": "added",
                "lastName": "removed",
                "email": "unchanged",
            },
            config=SheetConfig(
                name="Contacts",
                slug="contacts",
                fields=[
                    Property_String(
                        key="firstName",
                        label="First Name",
                    ),
                    Property_String(
                        key="lastName",
                        label="Last Name",
                    ),
                    Property_String(
                        key="email",
                        label="Email",
                    ),
                ],
                mapping_confidence_threshold=0.5,
            ),
        ),
        created_at=datetime.datetime.fromisoformat(
            "2023-01-01 00:00:00+00:00",
        ),
        created_by="us_usr_YOUR_ID",
    )
    """

    id: SnapshotId = pydantic.Field()
    """
    The ID of the Snapshot.
    """

    sheet_id: typing_extensions.Annotated[SheetId, FieldMetadata(alias="sheetId")] = pydantic.Field()
    """
    The ID of the Sheet.
    """

    label: typing.Optional[str] = pydantic.Field(default=None)
    """
    The title of the Snapshot.
    """

    summary: typing.Optional[SnapshotSummary] = pydantic.Field(default=None)
    """
    A summary of the Snapshot. This field is only available on the single get snapshot endpoint. It is not available for the list snapshots endpoint.
    """

    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    The time the Snapshot was created.
    """

    created_by: typing_extensions.Annotated[UserId, FieldMetadata(alias="createdBy")] = pydantic.Field()
    """
    The actor who created the Snapshot.
    """

    thread_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="threadId")] = pydantic.Field(
        default=None
    )
    """
    The ID of the thread associated with the snapshot.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
